Index: asterisk.c
===================================================================
--- asterisk.c	(revision 7814)
+++ asterisk.c	(working copy)
@@ -552,18 +617,19 @@
 
 static void *listener(void *unused)
 {
-	struct sockaddr_un sunaddr;
-	int s;
-	socklen_t len;
-	int x;
-	int flags;
-	struct pollfd fds[1];
 	pthread_attr_t attr;
+
 	pthread_attr_init(&attr);
 	pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);
+
 	for(;;) {
-		if (ast_socket < 0)
-			return NULL;
+		struct sockaddr_un sunaddr;
+		struct pollfd fds[1];
+		int s, x, flags;
+		socklen_t len;
+
+		if (ast_socket < 0)	/* quit_handler() will close and reset ast_socket asynchronously */
+			break;
 		fds[0].fd = ast_socket;
 		fds[0].events= POLLIN;
 		s = poll(fds, 1, -1);
@@ -577,37 +643,37 @@
 		if (s < 0) {
 			if (errno != EINTR)
 				ast_log(LOG_WARNING, "Accept returned %d: %s\n", s, strerror(errno));
-		} else {
-			for (x=0;x<AST_MAX_CONNECTS;x++) {
-				if (consoles[x].fd < 0) {
-					if (socketpair(AF_LOCAL, SOCK_STREAM, 0, consoles[x].p)) {
-						ast_log(LOG_ERROR, "Unable to create pipe: %s\n", strerror(errno));
-						consoles[x].fd = -1;
-						fdprint(s, "Server failed to create pipe\n");
-						close(s);
-						break;
-					}
-					flags = fcntl(consoles[x].p[1], F_GETFL);
-					fcntl(consoles[x].p[1], F_SETFL, flags | O_NONBLOCK);
-					consoles[x].fd = s;
-					if (ast_pthread_create(&consoles[x].t, &attr, netconsole, &consoles[x])) {
-						ast_log(LOG_ERROR, "Unable to spawn thread to handle connection: %s\n", strerror(errno));
-						consoles[x].fd = -1;
-						fdprint(s, "Server failed to spawn thread\n");
-						close(s);
-					}
-					break;
-				}
-			}
-			if (x >= AST_MAX_CONNECTS) {
-				fdprint(s, "No more connections allowed\n");
-				ast_log(LOG_WARNING, "No more connections allowed\n");
-				close(s);
-			} else if (consoles[x].fd > -1) {
-				if (option_verbose > 2) 
-					ast_verbose(VERBOSE_PREFIX_3 "Remote UNIX connection\n");
-			}
+			continue;
 		}
+		for (x=0;x<AST_MAX_CONNECTS;x++)
+			if (consoles[x].fd < 0)	/* this is a free entry */
+				break;
+		if (x >= AST_MAX_CONNECTS) {
+			ast_log(LOG_WARNING, "No more connections allowed\n");
+			fdprint(s, "No more connections allowed\n");
+			close(s);
+			continue;
+		}
+		if (socketpair(AF_LOCAL, SOCK_STREAM, 0, consoles[x].p)) {
+			ast_log(LOG_ERROR, "Unable to create pipe: %s\n", strerror(errno));
+			fdprint(s, "Server failed to create pipe\n");
+			close(s);
+			continue;
+		}
+		flags = fcntl(consoles[x].p[1], F_GETFL);
+		fcntl(consoles[x].p[1], F_SETFL, flags | O_NONBLOCK);
+		consoles[x].fd = s;
+		if (ast_pthread_create(&consoles[x].t, &attr, netconsole, &consoles[x])) {
+			ast_log(LOG_ERROR, "Unable to spawn thread to handle connection: %s\n", strerror(errno));
+			consoles[x].fd = -1;
+			close(consoles[x].p[0]);
+			close(consoles[x].p[1]);
+			fdprint(s, "Server failed to spawn thread\n");
+			close(s);
+			continue;
+		}
+		if (option_verbose > 2) 
+			ast_verbose(VERBOSE_PREFIX_3 "Remote UNIX connection\n");
 	}
 	return NULL;
 }
